import boto3
from datetime import datetime
from .settings import settings
from .utils import _get_or_create_metadata_table, _create_s3_bucket

region = settings.AWS_REGION
bucket_name = settings.S3_BUCKET_NAME
dynamo_state_table_name = settings.DYNAMODB_STATE_TABLE_NAME

if settings.ENVIRONMENT != 'local':  # not gonna need this on local
    dynamodb = boto3.resource('dynamodb', region_name=region)
    s3 = boto3.resource('s3', region_name=region)


def _upload_metadata(data_type, source_company, item):
    """
    - check whether the table exists ? then create accordingly
    - create of update item to the table (make sure the item object has "date", "timestamp")
    """
    # get or create table
    table_name = f'{data_type}-{source_company}'
    table = _get_or_create_metadata_table(table_name)
    _ = table.put_item(Item=item)
    

def _upload_data_to_s3(object_in, filename):
    if bucket_name not in [bucket.name for bucket in s3.buckets.all()]:
        _ = _create_s3_bucket(bucket_name=bucket_name, region=settings.AWS_REGION)
    obj = s3.Object(bucket_name, filename)
    response = obj.put(Body=object_in)
    
    return response


def upload_to_datalake(object_in, data_type, source_company, file_name):
    # upload object to S3
    now = datetime.now()
    date = now.date()
    # uid = hashing(now)
    # ts = int(now.timestamp() * 1000)

    key = f"{source_company}-{data_type}/{file_name}"
    _ = _upload_data_to_s3(object_in, key)

    # upload meta data to DynamoDB
    item = {
        "date": str(date),
        "timestamp": str(now),
        'upload_duration': str(datetime.now() - now),
        's3_location': {
            "bucket_name": bucket_name,
            "file_name": key
        }
    }
    _upload_metadata(data_type, source_company, item)



import boto3
from datetime import datetime
import hashlib
from botocore.exceptions import ClientError

from .settings import settings

region = settings.AWS_REGION

if settings.ENVIRONMENT != 'local':  # not gonna need this on local
    dynamodb = boto3.resource('dynamodb', region_name=region)
    s3 = boto3.resource('s3', region_name=region)


def hashing(s):
    return int(hashlib.sha256(s.encode('utf-8')).hexdigest(), 16) % 10**8


# DynamoDB
def _create_dynamodb_table(table_name, partition_key, sort_key=None):
    attribute_definition = [
        {
            'AttributeName': partition_key,
            'AttributeType': 'S',
        }
    ]
    key_schema = [
                    {
                        'AttributeName': partition_key,  # 20211231
                        'KeyType': 'HASH',  # partition key
                    },
    ]
    if sort_key:
        # docs: https://docs.aws.amazon.com/amazondynamodb/latest/developerguide/HowItWorks.NamingRulesDataTypes.html
        attribute_definition.append(
            {
                'AttributeName': sort_key,  # format : 2015-12-21T17:42:34Z
                'AttributeType': 'S',
            }
        )
        key_schema.append(
            {
                'AttributeName': sort_key,
                'KeyType': 'RANGE',  # sort key
            }
        )

    try:
        table = dynamodb.create_table(
                TableName=table_name,
                AttributeDefinitions=attribute_definition,
                KeySchema=key_schema,
                BillingMode='PAY_PER_REQUEST',
            )

        # Wait until the table exists.
        table.wait_until_exists()

    except ClientError as e:
        print('dynamodb.create_table():', e)
        return

    return table


def _get_or_create_metadata_table(table_name):
    if table_name not in [table.name for table in dynamodb.tables.all()]:
        table = _create_dynamodb_table(table_name, partition_key="date", sort_key="timestamp")
    else: 
        table = dynamodb.Table(table_name)
    return table


def _get_or_create_state_table(table_name):
    if table_name not in [table.name for table in dynamodb.tables.all()]:
        table = _create_dynamodb_table(table_name, partition_key="consumer")
    else: 
        table = dynamodb.Table(table_name)
    return table


# S3
def _create_s3_bucket(bucket_name, region):
    # Create bucket
    try:
        bucket = s3.create_bucket(
            ACL='private',
            Bucket=bucket_name,
            CreateBucketConfiguration={
                'LocationConstraint': region
            }
        )
    except ClientError as e:
        print('s3.create_bucket():', e)
        return None
    return bucket

    
def str_to_datetime(datetime_str):
    return datetime.strptime(datetime_str, '%Y-%m-%d %H:%M:%S.%f')

# metadata >>  n table
#     partition >> date
#     sortkey >> sortkey : timestamp + batch_id

# status-progress >> only 1 table
#     partition >> location-tiger-invoice , location-thaipost-invoice, matching-tiger-invoice
#     sortkey >> datetime
#     field >> 
#         - sortkey : timestamp + batch_id



from dns.resolver import resolve

from .settings import settings


def resolve_srv(url):
    answers = resolve(url, 'SRV')
    record = answers[0]
    host = record.target.to_text().rstrip('.')
    port = record.port
    a_url = f'http://{host}:{port}'
    return a_url


class Record:
    def __init__(self, name: str, type_: str = 'srv'):
        self.name: str = name
        self.type_: str = type_

    def __get__(self, instance: 'ServiceUrl', owner=None):
        if instance.environment == 'local':
            service_name = self.name.split('-')[0]
            return settings.DEV_API_GATEWAY_ENDPOINT + f'/api/{service_name}'

        if self.type_ == 'srv':
            return resolve_srv(f'{self.name}.lc-{instance.environment}')
        raise NotImplementedError()


class ServiceUrl:
    def __init__(self, environment):
        self.environment = environment

    matching = Record('matching-service', 'srv')
    tms = Record('tms-service', 'srv')
    location = Record('location-service', 'srv')
    pricing = Record('pricing-service', 'srv')


service_url = ServiceUrl(settings.ENVIRONMENT)
